# -*- coding: utf-8 -*-

from openerp import models, fields, api, exceptions, _
from openerp.osv import osv
from datetime import datetime
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT
from lxml import etree
from suds.client import Client
import re
import logging

_logger = logging.getLogger(__name__)


class account_invoice(models.Model):
    _inherit = 'account.invoice'

    sent_to_ws = fields.Boolean('Sent to WS?')

    @api.multi
    def send_data(self, xml):
        self.ensure_one()
        ws_url = (self.env['ir.config_parameter'].sudo().get_param('financing.ws.url') or 'http://servicioscpz.credipaz.com/wsMIL/wsMIL.asmx') + '?WSDL'
        ws_client = Client(ws_url)
        try:
            ws_response = ws_client.service.RegistrarVentaMil(sUsuario='odoo', sPassword='OdooP4sswd', sPin='203', xmlVenta=xml)
            import pprint; pprint.pprint(xml)
            m = re.match(r".*resultado=.(?P<resultado>\w*)", ws_response)
            assert m, _('Cannot match Credipaz Response')
        except Exception as e:
            _logger.warning('<<<DATA SENT TO CREDIPAZ>>>\n%s' % ws_client.last_sent())
            _logger.warning('<<<DATA RECEIVED FROM CREDIPAZ>>>\n%s' % ws_client.last_received())
            raise exceptions.ValidationError(_('Problem while sending data to credipaz: %s') % e)
        else:
            if m:
                retcode = m.groupdict().get('resultado')
                if retcode == 'OK':
                    self.sent_to_ws = True
                    self.message_post(body=_('Invoice informed to credipaz WS.'))
                else:
                    raise exceptions.ValidationError(_('Cannot send the invoice to WS'))

    @api.multi
    def inform_ws(self):
        for inv in self:
            if not self.sent_to_ws:
                order = self.env['pos.order'].search([('invoice_id', '=', inv.id)])
                if order:
                    ws_data_order = order.collect_data_for_ws()
                    _logger.debug('Data from pos.order\n%s' % ws_data_order)
                    # ws_data_pick = order.picking_id.collect_data_for_ws()
                    # _logger.debug('Data from stock.picking\n%s' % ws_data_pick)
                    ws_data_inv = inv.collect_data_for_ws()
                    _logger.debug('Data from account.invoice\n%s' % ws_data_inv)
                    # ws_data_order.update(ws_data_pick)
                    ws_data_order.update(ws_data_inv)
                    xml_ready_to_fly = inv.prepare_xml_for_ws(ws_data_order)
                    inv.send_data(xml_ready_to_fly)
                else:
                    pass

    @api.model
    def prepare_xml_for_ws(self, data):
        root = etree.Element('Operacion')
        # Fecha
        fecha_el = etree.SubElement(root, "Fecha")
        fecha_el2 = etree.SubElement(fecha_el, "Fecha")
        fecha_el2.text = data.get('date', '')
        # Hora
        hora_el = etree.SubElement(root, "Hora")
        hora_el2 = etree.SubElement(hora_el, "Hora")
        hora_el2.text = data.get('time', '')
        # Lugar: Sucursal, Terminal, Operador
        lugar_el = etree.SubElement(root, "Lugar")
        sucursal_el = etree.SubElement(lugar_el, "Sucursal")
        terminal_el = etree.SubElement(lugar_el, "Terminal")
        operador_el = etree.SubElement(lugar_el, "Operador")
        sucursal_el.text = data.get('suc_code', '')  # Numero
        terminal_el.text = data.get('terminal_id', '')  # 'PC07SU02'
        operador_el.text = data.get('usuario', '')  # 'CCIRINO'
        # Venta: TipoOperacion, DocumentoComprador, IdVenta, FormaPago, PrecioVenta, DescuentoAplicado, AgenteVentas
        venta_el = etree.SubElement(root, "Venta")
        TipoOperacion_el = etree.SubElement(venta_el, "TipoOperacion")
        TipoOperacion_el.text = data.get('type_operation', '')
        DocumentoComprador_el = etree.SubElement(venta_el, "DocumentoComprador")
        DocumentoComprador_el.text = data.get('customer_document', '')
        IdVenta_el = etree.SubElement(venta_el, "IdVenta")
        IdVenta_el.text = '3'  # TODO [ssaid@24/08/2017]: what is this?
        FormaPago_el = etree.SubElement(venta_el, "FormaPago")
        FormaPago_el.text = data.get('credipaz_payment_code', '')
        PrecioVenta_el = etree.SubElement(venta_el, "PrecioVenta")
        PrecioVenta_el.text = data.get('amount_total', '')
        DescuentoAplicado_el = etree.SubElement(venta_el, "DescuentoAplicado")
        DescuentoAplicado_el.text = '0'
        AgenteVentas_el = etree.SubElement(venta_el, "AgenteVentas")
        AgenteVentas_el.text = 'CCIRINO'
        # Productos
        productos_el = etree.SubElement(root, "Productos")
        for prod in data.get('products', []):
            producto_el = etree.SubElement(productos_el, "Producto")
            Codigo_el = etree.SubElement(producto_el, "Codigo")
            Codigo_el.text = prod.get('code', '')
            Descripcion_el = etree.SubElement(producto_el, "Descripcion")
            Descripcion_el.text = prod.get('desc', '')
            NumeroSerie_el = etree.SubElement(producto_el, "NumeroSerie")
            NumeroSerie_el.text = prod.get('serial_nr', '')
            PrecioUnitario_el = etree.SubElement(producto_el, "PrecioUnitario")
            PrecioUnitario_el.text = prod.get('price_unit', '')
        # Formas de pago
        formaspago_el = etree.SubElement(root, "FormasDePago")
        for pago in data.get('payments', []):
            fp_el = etree.SubElement(formaspago_el, "FormaDePago")
            TipoPago_el = etree.SubElement(fp_el, "TipoPago")
            TipoPago_el.text = pago.get('type', '')
            ImporteCantidad_el = etree.SubElement(fp_el, "ImporteCantidad")
            ImporteCantidad_el.text = pago.get('amount', '')
            Plazo_el = etree.SubElement(fp_el, "Plazo")
            Plazo_el.text = pago.get('term', '')
            Capital_el = etree.SubElement(fp_el, "Capital")
            Capital_el.text = pago.get('capital', '')
            MarcaTarjeta_el = etree.SubElement(fp_el, "MarcaTarjeta")
            MarcaTarjeta_el.text = pago.get('brand_card', '')
            CuentaTarjeta_el = etree.SubElement(fp_el, "CuentaTarjeta")
            CuentaTarjeta_el.text = pago.get('account_card', '')
            MiembroTarjeta_el = etree.SubElement(fp_el, "MiembroTarjeta")
            MiembroTarjeta_el.text = pago.get('member', '')
            PANTarjeta_el = etree.SubElement(fp_el, "PANTarjeta")
            PANTarjeta_el.text = pago.get('num_card', '')
        # Comprador
        comprador_el = etree.SubElement(root, "Comprador")
        Nombre_el = etree.SubElement(comprador_el, "Nombre")
        Nombre_el.text = data.get('customer_name', '')
        TipoDoc_el = etree.SubElement(comprador_el, "TipoDoc")
        TipoDoc_el.text = data.get('customer_document_type', '')
        Documento_el = etree.SubElement(comprador_el, "Documento")
        Documento_el.text = data.get('customer_document', '')

        # pretty string
        s = etree.tostring(root, pretty_print=True)
        print s
        return s

    @api.multi
    def collect_data_for_ws(self):
        self.ensure_one()
        res = {}
        date_dt = datetime.strptime(self.create_date, DEFAULT_SERVER_DATETIME_FORMAT)
        res['date'] = date_dt.strftime('%Y/%m/%d')
        res['time'] = date_dt.strftime('%H:%M')
        res['suc_code'] = self.pos_ar_id.name  # Numero
        res['terminal_id'] = self.pos_ar_id.shop_id.name  # Nombre
        res['usuario'] = self.user_id.name
        res['type_operation'] = 'VTA' if self.type == 'out_invoice' else 'DEV'
        res['customer_document'] = self.partner_id.vat
        res['customer_document_type'] = self.partner_id.document_type_id.name or ''
        res['customer_name'] = self.partner_id.name
        res['voucher_reference'] = self.internal_number
        res['amount_total'] = str(self.amount_total)
        dto_amount = sum(self.invoice_line.mapped(lambda x: (x.price_unit * (x.discount / float(100))) * x.quantity))
        res['discount_applied'] = str(dto_amount)
        return res

    @api.multi
    def invoice_validate(self):
        res = super(account_invoice, self).invoice_validate()
        for inv in self:
            if inv.type in ('out_invoice', 'out_refund'):
                order = self.env['pos.order'].search([('invoice_id', '=', inv.id)])
                # query = "SELECT ai.id invoice_id, po.id order_id, sp.id picking_id FROM pos_order po LEFT JOIN account_invoice ai ON po.invoice_id = ai.id JOIN stock_picking sp ON sp.id = po.picking_id WHERE ai.id=%s" % inv.id
                # self._cr.execute(query)
                # res = self._cr.dictfetchall()  # [{...}]
                if order:  # Collect data and send it to ws
                    ws_data = order.collect_data_for_ws()
                    # TODO [ssaid@24/08/2017]: SEND IT
                else:
                    # TODO [ssaid@24/08/2017]: Handle when there is no pos.order
                    pass
        return res


class pos_order(models.Model):
    _inherit = 'pos.order'

    def action_invoice(self, cr, uid, ids, context=None):
        """
        Override all the function cause, the onchange partner does not handle context and thus we cannot set the correct pos_ar_id
        """
        inv_ref = self.pool.get('account.invoice')
        inv_line_ref = self.pool.get('account.invoice.line')
        product_obj = self.pool.get('product.product')
        inv_ids = []

        for order in self.pool.get('pos.order').browse(cr, uid, ids, context=context):
            if order.invoice_id:
                inv_ids.append(order.invoice_id.id)
                continue

            if not order.partner_id:
                raise osv.except_osv(_('Error!'), _('Please provide a partner for the sale.'))

            acc = order.partner_id.property_account_receivable.id
            inv = {
                'name': order.name,
                'origin': order.name,
                'account_id': acc,
                'journal_id': order.sale_journal.id or None,
                'type': 'out_invoice',
                'reference': order.name,
                'partner_id': order.partner_id.id,
                'comment': order.note or '',
                'currency_id': order.pricelist_id.currency_id.id, # considering partner's sale pricelist's currency
            }
            onchange_res_value = inv_ref.onchange_partner_id(cr, uid, [], 'out_invoice', order.partner_id.id)['value']
            pos_pool = self.pool.get('pos.ar')
            pos_ar_name = order.session_id.config_id.pos_ar_name
            if not pos_ar_name:
                e_title = _('Missing PosArName')
                e_msg = _('In the configuration of pos, please set the pos.ar name.')
                raise exceptions.except_orm(e_title, e_msg)
            pos = pos_pool.search(cr, uid, [('denomination_id', '=', onchange_res_value['denomination_id']), ('name', '=', pos_ar_name)], order='priority asc', limit=1)
            value = {
                'pos_ar_id': pos and pos[0] or False,
            }
            onchange_res_value.update(value)
            import pprint; pprint.pprint(onchange_res_value)
            inv.update(onchange_res_value)
            if not inv.get('account_id', None):
                inv['account_id'] = acc
            inv_id = inv_ref.create(cr, uid, inv, context=context)

            self.write(cr, uid, [order.id], {'invoice_id': inv_id, 'state': 'invoiced'}, context=context)
            inv_ids.append(inv_id)
            for line in order.lines:
                inv_line = {
                    'invoice_id': inv_id,
                    'product_id': line.product_id.id,
                    'quantity': line.qty,
                }
                inv_name = product_obj.name_get(cr, uid, [line.product_id.id], context=context)[0][1]
                inv_line.update(inv_line_ref.product_id_change(cr, uid, [],
                                                               line.product_id.id,
                                                               line.product_id.uom_id.id,
                                                               line.qty, partner_id = order.partner_id.id)['value'])
                if not inv_line.get('account_analytic_id', False):
                    inv_line['account_analytic_id'] = \
                        self._prepare_analytic_account(cr, uid, line,
                                                       context=context)
                inv_line['price_unit'] = line.price_unit
                inv_line['discount'] = line.discount
                inv_line['name'] = inv_name
                inv_line['invoice_line_tax_id'] = [(6, 0, inv_line['invoice_line_tax_id'])]
                inv_line_ref.create(cr, uid, inv_line, context=context)
            inv_ref.button_reset_taxes(cr, uid, [inv_id], context=context)
            self.signal_workflow(cr, uid, [order.id], 'invoice')
            inv_ref.signal_workflow(cr, uid, [inv_id], 'validate')

        if not inv_ids: return {}

        mod_obj = self.pool.get('ir.model.data')
        res = mod_obj.get_object_reference(cr, uid, 'account', 'invoice_form')
        res_id = res and res[1] or False
        return {
            'name': _('Customer Invoice'),
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [res_id],
            'res_model': 'account.invoice',
            'context': "{'type':'out_invoice'}",
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'current',
            'res_id': inv_ids and inv_ids[0] or False,
        }
