# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2017-TODAY Eynes (<http://www.e-mips.com.ar>)
#
#    This is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import api, fields, models
from openerp.addons import decimal_precision as dp
from datetime import datetime
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT
import logging

_logger = logging.getLogger(__name__)


class POSOrder(models.Model):
    _inherit = 'pos.order'

    quota_qty = fields.Integer(string='Quota Qty.', default=1)
    quota_amount = fields.Float(string='Quota Amount', digits=dp.get_precision('Product UoS'))

    @api.model
    def get_card_institution(self, card_number):
        if card_number.startswith('3'):
            return 'AMEX'
        elif card_number.startswith('4'):
            return 'VISA'
        elif card_number.startswith('5'):
            return 'MASTERCARD'
        else:
            return '_'

    @api.model
    def add_payment(self, order_id, data):
        res = super(POSOrder, self).add_payment(order_id, data)  # res is statement_id
        aj = self.env['account.journal'].browse(data['journal'])
        if aj.pos_payment_type == 'financial':  # write card number
            absl = self.env['account.bank.statement.line'].search([
                ('pos_statement_id', '=', order_id),
                '&',
                ('journal_id', '=', data.get('journal')),
                ('amount', '=', data.get('amount')),
            ])
            card_number = data.get('card_number')
            card_issuer = self.get_card_institution(card_number)
            absl.card_number = '%s: %s' % (card_issuer, card_number)
        return res

    @api.multi
    def collect_data_for_ws(self):
        self.ensure_one()
        res = {}
        # Now process data from order
        am_cc = sum(self.statement_ids.filtered(lambda x: x.journal_id.credipaz_card is True).mapped('amount'))
        res['amount_cc'] = am_cc
        am_pt = sum(self.statement_ids.filtered(lambda x: x.journal_id.use_points is True).mapped('amount'))
        financial_statements = self.statement_ids.filtered(lambda x: x.journal_id.pos_payment_type == 'financial')
        am_all_financial_capital = sum(financial_statements.mapped('amount'))
        am_total_financed = self.quota_qty * self.quota_amount
        res['amount_pt'] = am_pt
        res['qty_quota'] = self.quota_qty
        res['amount_quota'] = self.quota_amount
        res['credipaz_payment_code'] = self.statement_ids.get_credipaz_payment_code()
        res['payments'] = []
        for payment in self.statement_ids:
            if payment.journal_id.pos_payment_type == 'financial':
                card_split = payment.card_number.split(': ')
                card_name = card_split[0]
                card_number = card_split[1]
                account_card = self.partner_id.ref or '' if payment.journal_id.credipaz_card is True else ''
                member = self.partner_id.credipaz_card_additional if payment.journal_id.credipaz_card is True else ''
                term = self.quota_qty
                capital = payment.amount
                amount = (payment.amount / am_all_financial_capital) * am_total_financed
            elif payment.journal_id.pos_payment_type == 'cash':
                if payment.journal_id.use_points:
                    member = self.partner_id.credipaz_card_additional or ''
                    account_card = self.partner_id.ref
                    cp_card = self.statement_ids.filtered('journal_id.credipaz_card')
                    term = 0
                    amount = payment.qty_points
                    capital = payment.qty_points
                    if cp_card:
                        card_split = cp_card[0].card_number.split(': ')
                        card_name = card_split[0]
                        card_number = card_split[1]
                    else:
                        card_name = 'CABAL'
                        card_number = ''
                else:  # cash
                    card_name = ''
                    card_number = ''
                    account_card = ''
                    term = ''
                    member = ''
                    amount = payment.amount
                    capital = payment.amount
            else:  # Should never enter here!
                print 'never enter here!'
                card_name = ''
                card_number = ''
                account_card = ''
                member = ''
                term = 0
                amount = payment.amount
            payment_data = {
                'num_card': card_number,
                'brand_card': card_name,
                'account_card': account_card,
                'type': payment.journal_id.name,
                'amount': str(amount),
                'capital': str(capital),
                'term': str(term),
                'member': member,
            }
            res['payments'].append(payment_data)
        res['products'] = []
        for line in self.lines:
            prod_data = {
                'code': line.product_id.default_code or '',
                'desc': line.product_id.name or '',
                'serial_nr': line.prodlot_id.name or '',
                'price_unit': str(line.price_subtotal_incl),
            }
            res['products'].append(prod_data)
        import pprint; pprint.pprint(res)
        return res

    @api.model
    def create_from_ui(self, orders):
        res = super(POSOrder, self).create_from_ui(orders)
        # orders = self.browse(res)
        # for order in orders:
        #     order.collect_data_for_ws()
        return res

    @api.model
    def _order_fields(self, ui_order):
        res = super(POSOrder, self)._order_fields(ui_order)
        res["quota_qty"] = ui_order["quota_qty"]
        res["quota_amount"] = ui_order["quota_amount"]
        return res

    @api.model
    def _payment_fields(self, ui_paymentline):
        res = super(POSOrder, self)._payment_fields(ui_paymentline)
        res.update({
            'card_number': ui_paymentline['card_number'],
        })
        return res


class PosPriceListConfig(models.Model):
    _inherit = 'pos.config'

    display_price_with_taxes = fields.Boolean(
        string='Price With Taxes',
        help="Display Prices with taxes on POS"
    )
    pos_ar_name = fields.Char('Point of Sale Name')
    stock_location_transit_id = fields.Many2one('stock.location', string='Stock Location(Transit)', domain=[('usage', '=', 'internal')], required=True)


class pos_session(models.Model):
    _inherit = 'pos.session'

    def wkf_action_close(self, cr, uid, ids, context=None):
        po_obj = self.pool['pos.order']
        aml_obj = self.pool['account.move.line']

        # Call regular workflow
        res = super(pos_session, self).wkf_action_close(
            cr, uid, ids, context=context)

        # Get All Pos Order invoiced during the current Sessions
        po_ids = po_obj.search(cr, uid, [
            ('session_id', 'in', ids),
            ('invoice_id', '!=', False),
        ], context=context)
        for po in po_obj.browse(cr, uid, po_ids, context=context):
            # We're searching only account Invoices that has been payed
            # In Point Of Sale
            #if not po.invoice_id.forbid_payment:
            #    continue

            # Search all move Line to reconcile in Sale Journal
            aml_sale_ids = []
            aml_sale_total = 0
            for aml in po.invoice_id.move_id.line_id:
                if (aml.partner_id.id == po.partner_id.id and
                        aml.account_id.type == 'receivable'):
                    aml_sale_ids.append(aml.id)
                    aml_sale_total += aml.debit - aml.credit

            aml_payment_ids = []
            aml_payment_total = 0
            # Search all move Line to reconcile in Payment Journals
            abs_ids = list(set([x.statement_id.id for x in po.statement_ids]))
            aml_ids = aml_obj.search(cr, uid, [
                ('statement_id', 'in', abs_ids),
                ('partner_id', '=', po.partner_id.id),
                ('reconcile_id', '=', False)], context=context)
            for aml in aml_obj.browse(
                    cr, uid, aml_ids, context=context):
                if (aml.account_id.type == 'receivable'):
                    aml_payment_ids.append(aml.id)
                    aml_payment_total += aml.debit - aml.credit

            # Try to reconcile
            if aml_payment_total != - aml_sale_total:
                # Unable to reconcile
                _logger.warning(
                    "Unable to reconcile the payment of %s #%s."
                    "(partner : %s), %s != %s" % (
                        po.name, po.id, po.partner_id.name, aml_payment_total, aml_sale_total))
            else:
                aml_obj.reconcile(
                    cr, uid, aml_payment_ids + aml_sale_ids, 'manual',
                    False, False, False, context=context)

        return res
