# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2017-TODAY Eynes (<http://www.e-mips.com.ar>)
#
#    This is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

#import itertools
import json

from openerp import _, api, fields, models


class ProductProduct(models.Model):
    _inherit = 'product.product'

    @api.depends("financings")
    def compute_financing_tables(self):
        table_template = """
        <table class="tooltip_table">
            <tr>
                <th class="tooltip_table_header">{}</th>
                <th class="tooltip_table_header">{}</th>
                <th class="tooltip_table_header">{}</th>
                <th class="tooltip_table_header">{}</th>
                <th class="tooltip_table_header">{}</th>
            </tr>
            %s
        </table>
        """.format(_("Quota"), _("Quota Amount"), _("Total"), _("A.P.R."), _("T.F.C."))

        data_template = """
        <tr>
            <td class="tooltip_table_data">{f.financing_config_id.quota_qty}</td>
            <td class="tooltip_table_data">{f.quota_amount_taxed:.2f}</td>
            <td class="tooltip_table_data">{f.total_taxed:.2f}</td>
            <td class="tooltip_table_data">{f.apr:.3f}</td>
            <td class="tooltip_table_data">{f.tfc:.3f}</td>
        </tr>
        """

        no_financing_msg = _('This product is not properly configured!')
        empty_json = json.dumps(no_financing_msg)
        for product in self:
            if not product.financings:
                product.financing_tables = empty_json
                continue

            # Build table rows
            rows = {}
            for financing in product.financings:
                rows.setdefault(financing.pos_config_id.id, {}).setdefault(
                    financing.customer_type, []).append(data_template.format(f=financing))

            # Join rows with the table header
            tables = {}
            for pos_config_id, data in rows.items():
                for customer_type, row in data.items():
                    table = table_template % "\n".join(row)
                    tables.setdefault(pos_config_id, {})[customer_type] = table

            #product.financing_tables = ";".join(itertools.chain(*tables.items()))
            product.financing_tables = json.dumps(tables)

    financings = fields.One2many('product.financing', 'product_id', string='Financings')
    financing_tables = fields.Text(string='Financing Tables', compute="compute_financing_tables")
