/******************************************************************************
 *    Financing - Financing on Odoo
 *    Copyright (C) 2017 Eynes
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

function financing_db(instance, module) {

    var PosDBParent = module.PosDB;
    module.PosDB = module.PosDB.extend({

        init: function (options) {
            options = options || {};
            this._super(options);
            this.financing_by_product_id = {};
            this.table_by_product_id = {};
            this.financing_configs = {};
        },
        
        add_partners: function(partners){  // prevent pos from viewing false as string in partner creation
            var self = this;
            for (var i = 0, len = partners.length; i < len; i++) {
                p = partners[i];
                for (var key in p){
                    if (p.hasOwnProperty(key)) {
                        if (p[key] === false) {
                            p[key] = "";
                        }
                    }
                }
            }
            res = this._super(partners);
            return res
        },
        add_products_with_pos: function (products, pos_model) {
            this.add_products(products);
            if (!products instanceof Array) {
                products = [products];
            }
            tax_obj = {};
            for (var i = 0, len = pos_model.taxes.length; i < len; i++) {
                tax_obj[pos_model.taxes[i].id] = pos_model.taxes[i];
            }
            for(var i = 0, len = products.length; i < len; i++){
                var product = products[i];
                var product_id = product.id;
                this.table_by_product_id[product_id] = { false: "" };
                var tables = product.financing_tables;

                var tax_to_apply = tax_obj[product.taxes_id[0]];
                product.price_with_taxes = product.list_price * (1 + tax_to_apply.amount)


                if (!tables) { continue };

                //var splits = tables.split(";");
                //if (tables.indexOf(";") < 0 || splits.length <= 1) { continue };
                //var tables = JSON.parse(product.financing_tables[pos_model.config.id]);
                var tables_obj = JSON.parse(tables);
                var config_id = pos_model.config.id.toString();
                if ( !tables_obj.hasOwnProperty(config_id) ) { continue };
                this.table_by_product_id[product_id] = tables_obj[config_id];

                //for (var j = 0; j < splits.length; j++) {
                //    //var customer_type = splits[j].trim(); 
                //    //var table = splits[j + 1].trim(); 
                //    j++;
                //}
            }
        },

        add_financings: function (financings) {
            // build an useful financing map table as follows: product_id,customer_type -> [financings]
            if (!(financings instanceof Array)) {
                financings = [financings];
            }
            //console.log(this);

            for (var i=0; i < financings.length; i++) {
                financing = financings[i];
                // get product_id form the financing
                var product_id = financing.product_id[0];
                if (!this.financing_by_product_id.hasOwnProperty(product_id)) { 
                    // set product_id default map
                    this.financing_by_product_id[product_id] = {};
                }

                var fbp = this.financing_by_product_id[product_id];
                if (!fbp.hasOwnProperty(financing.customer_type)) { 
                    // set customer_type default array
                    this.financing_by_product_id[product_id][financing.customer_type] = [];
                }

                // append current financing to the array
                this.financing_by_product_id[product_id][financing.customer_type].push(financing);
            }
        },

        add_financing_configs: function (financing_configs, pos_model) {
            if (!(financing_configs instanceof Array)) {
                financing_configs = [financing_configs];
            }

            for (var i=0; i < financing_configs.length; i++) {
                config = financing_configs[i];
                if (!this.financing_configs.hasOwnProperty(config.customer_type)) { 
                    // set customer_type default array
                    this.financing_configs[config.customer_type] = [];
                }

                this.financing_configs[config.customer_type][config.quota_qty] = config;
            }
            pos_model.set_enabled_quotas(financing_configs);
        },

        search_financing_by_quota_qty: function (product_id, customer_type, quota_qty) {
            var financings = this.financing_by_product_id[product_id][customer_type]
            for (var i = 0; i < financings.length; i++) {
                var financing = financings[i];
                if (financing.quota_qty === quota_qty) {
                    return financing;
                }
            }
            return null;
        }

    })
}
